-- ------------------------------------------------- --
-- ZoneHistory - (c) desp <Dees@Die Aldor-EU> , 2009 --
-- ------------------------------------------------- --

-- Vars --

ZH_Config = { ["silent"] = false, ["timeout"] = 0, ["lastArea"] = "?", ["autohide"] = 0 };
ZH_History = {};
ZH_Blacklist = {};

local ZH_CurrentArea = nil;
local ZH_CurrentAreaTimeEntered = nil;
local ZH_VisitedAreas = nil;

-- Event Handler --

function ZH_OnEvent()
	if(event == "ZONE_CHANGED_NEW_AREA") then
		ZH_OnAreaChanged();
	elseif(event == "PLAYER_LOGOUT") then
		ZH_OnLogout();
	elseif(event == "ADDON_LOADED" and arg1 == "ZoneHistory") then
		ZH_OnAddonLoaded();
	end
end

function ZH_OnAreaChanged()
	ZH_SaveCurrentArea();
	ZH_NewArea();
end

function ZH_OnAddonLoaded()
	ZH_UpdateDatabase();
	ZH_InitSlashCmds();
	
	ZH_NewArea();
	ZH_UpdateVisitedAreas();
end

function ZH_OnSlashCmd(msg)
	local cmd, scmd = ZH_GetSlashCmd(msg);

	if(cmd == "h" or cmd == "history") then
		if(scmd == "") then
			ZH_PrintVisitedAreas();
		else
			ZH_PrintAreaInfo(ZH_VisitedAreas[tonumber(scmd)]);
		end
	elseif(cmd == "DELETE") then
		if(scmd == "") then
			ZH_PrintVisitedAreas();
		else
			ZH_DeleteAreaHistory(tonumber(scmd));
		end
	elseif(cmd == "bladd") then
		ZH_AddToBlacklist(scmd);
	elseif(cmd == "blrem") then
		ZH_RemoveFromBlacklist(scmd);
	elseif(cmd == "bl" or cmd == "blacklist") then
		ZH_PrintBlacklist();
	elseif(cmd == "timeout") then
		ZH_SetTimeout(tonumber(scmd));
	elseif(cmd == "autohide") then
		ZH_SetAutohide(tonumber(scmd));
	elseif(cmd == "s" or cmd == "silent") then
		ZH_Config["silent"] = (not ZH_Config["silent"]);
		ZH_Print(ZH_ColorBlue("Silent Mode: ") .. tostring(ZH_Config["silent"]) .. ".");
	else
		ZH_PrintAvailableSlashCmds();
	end
end

function ZH_OnLogout()
	ZH_SaveCurrentArea();
	
	ZH_Config["lastArea"] = GetRealZoneText();
end

-- Area Related Functions --

function ZH_NewArea()
	local areaName = GetRealZoneText();
	
	if(areaName ~= "") then
		ZH_CurrentArea = ZH_CopyTable(ZH_History[areaName]);
		ZH_CurrentAreaTimeEntered = time();
		
		if(not ZH_CurrentArea) then
			ZH_CurrentArea = {
				["name"] = areaName,
				["stayedInSecs"] = 0,
				["left"] = 0,
				["numEntered"] = 0
			};

			ZH_History[areaName] = ZH_CopyTable(ZH_CurrentArea);
			ZH_UpdateVisitedAreas();
		end
		
		if(not ZH_Config["silent"] and not ZH_Blacklist[areaName]) then
			ZH_PrintAreaInfo(areaName);
		end
	end
end

function ZH_SaveCurrentArea()
	if(ZH_CurrentArea) then
		local stayedInSecs, now = ZH_StayedInSecs();
		
		ZH_CurrentArea["stayedInSecs"] = ZH_CurrentArea["stayedInSecs"] + stayedInSecs;
		
		if(stayedInSecs >= ZH_Config["timeout"]) then
			ZH_CurrentArea["left"] = now;
			
			if(ZH_Config["lastArea"] ~= ZH_CurrentArea["name"]) then
				ZH_CurrentArea["numEntered"] = ZH_CurrentArea["numEntered"] + 1;
			else
				ZH_Config["lastArea"] = "?";
			end
		end
		
		ZH_History[ZH_CurrentArea["name"]] = ZH_CurrentArea;
		
		ZH_CurrentArea = nil;
		ZH_CurrentAreaTimeEntered = nil;
	end
end

function ZH_PrintAreaInfo(name)
	local name, numEntered, lastVisited, stayedInSecs = ZH_GetAreaInfo(name);

	if(name ~= "") then
		ZH_Print(ZH_ColorBlue("## " .. name .. " (" .. numEntered .. ") ##"));

		if(lastVisited > 0) then
			ZH_Print(ZH_ColorBlue("Last Visit: ") .. date("%c", lastVisited) .. ".");
		end

		if(stayedInSecs > 0) then
			ZH_Print(ZH_ColorBlue("Time Spent: ") .. SecondsToTime(stayedInSecs) .. ".");
		end
	else
		ZH_Print("Unknown Zone.");
	end
end

function ZH_GetAreaInfo(name)
	if(not name) then
		return "", 0, 0, 0, false;
	end

	local areaIsCurrent = (ZH_CurrentArea and ZH_CurrentArea["name"] == name);
	
	if(areaIsCurrent) then
		data = ZH_CopyTable(ZH_CurrentArea);
		data["stayedInSecs"] = data["stayedInSecs"] + ZH_StayedInSecs();
		
		if(name ~= ZH_Config["lastArea"]) then
			data["numEntered"] = data["numEntered"] + 1;
		end
	else
		data = ZH_CopyTable(ZH_History[name]);
		
		if(not data) then
			return "", 0, 0, 0, false;
		end
	end
	
	return name, data["numEntered"], data["left"], data["stayedInSecs"], areaIsCurrent;
end

function ZH_PrintVisitedAreas()
	ZH_Print(ZH_ColorBlue("## Visited Areas ##"));

	for k, v in pairs(ZH_VisitedAreas) do
		ZH_Print("- " .. ZH_ColorBlue(k) .. " : " .. v);
	end
end

function ZH_UpdateVisitedAreas()
	local visitedAreas = {};
	
	for k, v in pairs(ZH_History) do
		if(not ZH_Blacklist[k] and v["stayedInSecs"] >= ZH_Config["autohide"]) then
			table.insert(visitedAreas, k);
		end
	end
	
	table.sort(visitedAreas);
	ZH_VisitedAreas = visitedAreas;
end

function ZH_DeleteAreaHistory(id)
	if(id) then
		local areaName = ZH_VisitedAreas[id];
		
		if(areaName) then
			ZH_Print("Deleted Area: " .. ZH_ColorBlue(areaName) .. ".");
			ZH_History[areaName] = nil;
			ZH_UpdateVisitedAreas();
		else
			ZH_Print("Area not found.");
		end
	end
end

function ZH_AddToBlacklist(name)
	if(name and name ~= "") then
		ZH_Blacklist[name] = true;
		ZH_Print("Added " .. ZH_ColorBlue(name) .. " to the Blacklist.");
		ZH_UpdateVisitedAreas();
	else
		ZH_Print("No name given.");
	end
end

function ZH_RemoveFromBlacklist(name)
	if(name and name ~= "") then
		ZH_Blacklist[name] = nil;
		ZH_Print("Removed " .. ZH_ColorBlue(name) .. " from the Blacklist.");
		ZH_UpdateVisitedAreas();
	else
		ZH_Print("No name given.");
	end
end

function ZH_PrintBlacklist()
	ZH_Print(ZH_ColorBlue("## Blacklist ##"));
	
	for k, v in pairs(ZH_Blacklist) do
		ZH_Print("- " .. k);
	end
end

function ZH_SetTimeout(sec)
	if(not sec) then
		sec = 180;
	end
	
	ZH_Config["timeout"] = sec;
	ZH_Print(ZH_ColorBlue("Timeout: ") .. sec .. "sec.");
end

function ZH_SetAutohide(mins)
	if(mins > 0) then
		ZH_Config["autohide"] = mins * 60;
		ZH_Print(ZH_ColorBlue("Autohide: ") .. mins .. "min.");
	else
		ZH_Config["autohide"] = 0;
		ZH_Print(ZH_ColorBlue("Autohide: ") .. "disabled.");
	end
	
	ZH_UpdateVisitedAreas();
end

-- Slash Cmd Related Funtions --

function ZH_PrintAvailableSlashCmds()
	ZH_Print(ZH_ColorBlue("Available Commands:") .. " {h,history (ID)} | {bl,blacklist} | {bladd (NAME)} | {blrem (NAME)} | {DELETE (ID)} | {timeout (SEC)} | {autohide (MIN)} | {s,silent}");
end

function ZH_InitSlashCmds()
	SlashCmdList["ZONE_HISTORY"] = ZH_OnSlashCmd;
	SLASH_ZONE_HISTORY1 = "/zonehistory";
	SLASH_ZONE_HISTORY2 = "/zh";
end

function ZH_GetSlashCmd(msg)
 	if(msg) then
 		local a, b, c = strfind(msg, "(%S+)");
		
 		if(a) then
 			return c, strsub(msg, b + 2);
 		else	
 			return "";
 		end
	end
end

-- Misc Functions --

function ZH_UpdateDatabase()
	if(ZH_History["_lastArea"]) then
		ZH_Config["lastArea"] = ZH_History["_lastArea"];
		ZH_History["_lastArea"] = nil;
	end
	
	if(not ZH_Config["timeout"]) then
		ZH_Config["timeout"] = 0;
	end
	
	if(not ZH_Config["autohide"]) then
		ZH_Config["autohide"] = 0;
	end
end

function ZH_StayedInSecs()
	local now = time();
	local stayedInSecs = now - ZH_CurrentAreaTimeEntered;

	return stayedInSecs, now;
end

function ZH_Print(msg)
	if(msg) then
		DEFAULT_CHAT_FRAME:AddMessage("|cff456789ZoneHistory:|r " .. msg);
	end
end

function ZH_ColorBlue(msg)
	return "|cffabcdef" .. msg .. "|r";
end

function ZH_CopyTable(tbl)
	if(not tbl) then
		return;
	end

	local result = {};
	
	for k, v in pairs(tbl) do
		result[k] = v;
	end
	
	return result;
end